@@ -243,19 +243,6 @@ trx_purge_add_update_undo_to_history(
 			hist_size + undo->size, MLOG_4BYTES, mtr);
 	}
 
-	/* Before any transaction-generating background threads or the
-	purge have been started, recv_recovery_rollback_active() can
-	start transactions in row_merge_drop_temp_indexes() and
-	fts_drop_orphaned_tables(), and roll back recovered transactions.
-	After the purge thread has been given permission to exit,
-	in fast shutdown, we may roll back transactions (trx->undo_no==0)
-	in THD::cleanup() invoked from unlink_thd(). */
-	ut_ad(srv_undo_sources
-	      || ((srv_startup_is_before_trx_rollback_phase
-		   || trx_rollback_or_clean_is_active)
-		  && purge_sys->state == PURGE_STATE_INIT)
-	      || (trx->undo_no == 0 && srv_fast_shutdown));
-
 	/* Add the log as the first in the history list */
 	flst_add_first(rseg_header + TRX_RSEG_HISTORY,
 		       undo_header + TRX_UNDO_HISTORY_NODE, mtr);
@@ -288,18 +275,37 @@ trx_purge_add_update_undo_to_history(
 	}
 }
 
-/**********************************************************************//**
-Frees an undo log segment which is in the history list. Cuts the end of the
-history list at the youngest undo log in this segment. */
+/** Remove undo log header from the history list.
+@param[in,out]	rseg_hdr	rollback segment header
+@param[in]	log_hdr		undo log segment header
+@param[in,out]	mtr		mini transaction. */
+static
+void
+trx_purge_remove_log_hdr(
+	trx_rsegf_t*	rseg_hdr,
+	trx_ulogf_t*	log_hdr,
+	mtr_t*		mtr)
+{
+	flst_remove(rseg_hdr + TRX_RSEG_HISTORY,
+		    log_hdr + TRX_UNDO_HISTORY_NODE, mtr);
+#ifdef HAVE_ATOMIC_BUILTINS
+	os_atomic_decrement_ulint(&trx_sys->rseg_history_len, 1);
+#else
+	mutex_enter(&trx_sys->mutex);
+	--trx_sys->rseg_history_len;
+	mutex_exit(&trx_sys->mutex);
+#endif
+}
+
+/** Frees an undo log segment which is in the history list. Removes the
+undo log hdr from the history list.
+@param[in,out]	rseg		rollback segment
+@param[in]	hdr_addr	file address of log_hdr */
 static
 void
 trx_purge_free_segment(
-/*===================*/
-	trx_rseg_t*	rseg,		/*!< in: rollback segment */
-	fil_addr_t	hdr_addr,	/*!< in: the file address of log_hdr */
-	ulint		n_removed_logs)	/*!< in: count of how many undo logs we
-					will cut off from the end of the
-					history list */
+	trx_rseg_t*	rseg,
+	fil_addr_t	hdr_addr)
 {
 	mtr_t		mtr;
 	trx_rsegf_t*	rseg_hdr;
@@ -363,16 +369,7 @@ trx_purge_free_segment(
 	history list: otherwise, in case of a database crash, the segment
 	could become inaccessible garbage in the file space. */
 
-	flst_cut_end(rseg_hdr + TRX_RSEG_HISTORY,
-		     log_hdr + TRX_UNDO_HISTORY_NODE, n_removed_logs, &mtr);
-
-#ifdef HAVE_ATOMIC_BUILTINS
-	os_atomic_decrement_ulint(&trx_sys->rseg_history_len, n_removed_logs);
-#else
-	mutex_enter(&trx_sys->mutex);
-	trx_sys->rseg_history_len -= n_removed_logs;
-	mutex_exit(&trx_sys->mutex);
-#endif /* HAVE_ATOMIC_BUILTINS */
+	trx_purge_remove_log_hdr(rseg_hdr, log_hdr, &mtr);
 
 	do {
 
@@ -414,7 +411,6 @@ trx_purge_truncate_rseg_history(
 	page_t*		undo_page;
 	trx_ulogf_t*	log_hdr;
 	trx_usegf_t*	seg_hdr;
-	ulint		n_removed_logs	= 0;
 	mtr_t		mtr;
 	trx_id_t	undo_trx_no;
 
@@ -452,19 +448,6 @@ loop:
 				hdr_addr.boffset, limit->undo_no);
 		}
 
-#ifdef HAVE_ATOMIC_BUILTINS
-		os_atomic_decrement_ulint(
-			&trx_sys->rseg_history_len, n_removed_logs);
-#else
-		mutex_enter(&trx_sys->mutex);
-		trx_sys->rseg_history_len -= n_removed_logs;
-		mutex_exit(&trx_sys->mutex);
-#endif /* HAVE_ATOMIC_BUILTINS */
-
-		flst_truncate_end(rseg_hdr + TRX_RSEG_HISTORY,
-				  log_hdr + TRX_UNDO_HISTORY_NODE,
-				  n_removed_logs, &mtr);
-
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 
@@ -473,7 +456,6 @@ loop:
 
 	prev_hdr_addr = trx_purge_get_log_from_hist(
 		flst_get_prev_addr(log_hdr + TRX_UNDO_HISTORY_NODE, &mtr));
-	n_removed_logs++;
 
 	seg_hdr = undo_page + TRX_UNDO_SEG_HDR;
 
@@ -485,10 +467,14 @@ loop:
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 
-		trx_purge_free_segment(rseg, hdr_addr, n_removed_logs);
+		/* calls the trx_purge_remove_log_hdr()
+		inside trx_purge_free_segment(). */
+		trx_purge_free_segment(rseg, hdr_addr);
 
-		n_removed_logs = 0;
 	} else {
+		/* Remove the log hdr from the rseg history. */
+		trx_purge_remove_log_hdr(rseg_hdr, log_hdr, &mtr);
+
 		mutex_exit(&(rseg->mutex));
 		mtr_commit(&mtr);
 	}
@@ -604,7 +590,7 @@ trx_purge_rseg_get_next_history_log(
 				"InnoDB: but its length is still"
 				" reported as %lu! Make a detailed bug\n"
 				"InnoDB: report, and submit it"
-				" to https://jira.mariadb.org/\n",
+				" to http://bugs.mysql.com\n",
 				(ulong) trx_sys->rseg_history_len);
 			ut_ad(0);
 		}
